import Mousetrap from 'mousetrap';
import { useEffect } from 'react';

export interface KeyMapItem {
  name: string;
  sequences: string[];
  action: 'keydown' | 'keyup' | 'keypress';
}

export type KeyMap = Record<string, KeyMapItem>;

export type Handlers<TKeyMap extends KeyMap> = {
  [key in keyof TKeyMap]: (event: KeyboardEvent, shortcut: string) => void;
};

interface Props<TKeyMap extends KeyMap> {
  children?: JSX.Element;
  keyMap: TKeyMap;
  handlers: Handlers<TKeyMap>;
}

export default function GlobalHotKeys<TKeyMap extends KeyMap>(
  props: Props<TKeyMap>,
): JSX.Element {
  const { children, keyMap, handlers } = props;
  useEffect(() => {
    for (const key of Object.keys(keyMap)) {
      const { sequences, action } = keyMap[key];
      const handler = handlers[key];
      Mousetrap.bind(sequences, handler, action);
    }

    return () => {
      for (const key of Object.keys(keyMap)) {
        const { sequences, action } = keyMap[key];
        Mousetrap.unbind(sequences, action);
      }
    };
  });

  return children || <></>;
}
