import { i18n } from '@lingui/core';
import { I18nProvider } from '@lingui/react';
import { ConfigProvider } from 'antd';
import { Locale } from 'antd/es/locale';
import enUS from 'antd/es/locale/en_US';
import viVN from 'antd/es/locale/vi_VN';
import { ANT_PREFIX_CLASS } from 'consts';
import 'dayjs/locale/en';
import 'dayjs/locale/vi';
import { dynamicActivate, getLocale } from 'i18n';
import * as React from 'react';
import { colors, shapes, typography } from 'theme/opus';
import { useGlobalState, useLocalStorage } from 'utils/hooks';

const antdLocales: Record<string, Locale> = {
  en: enUS,
  vi: viVN,
};

interface InternationalizationProps {
  children: React.ReactNode;
}

const theme: React.ComponentProps<typeof ConfigProvider>['theme'] = {
  token: {
    // colorPrimary: colors.primary,
    colorBorder: colors.border.default,
    // fontFamily: typography.fontFamily,
    fontSize: typography.fontSize.base,
    // fontSizeHeading1: typography.fontSize.h1,
    // fontSizeHeading2: typography.fontSize.h2,
    // fontSizeHeading3: typography.fontSize.h3,
    // fontSizeHeading4: typography.fontSize.h4,
    // fontSizeHeading5: typography.fontSize.h5,
    fontWeightStrong: typography.fontWeight.bold,
    borderRadius: shapes.borderRadius.button,
    colorLink: typography.fontColor.hyperlink,
    colorTextBase: typography.fontColor.default,
    // lineHeight: typography.lineHeight.bodyText,
    // lineHeightHeading1: typography.lineHeight.title,
    // lineHeightHeading2: typography.lineHeight.title,
    // lineHeightHeading3: typography.lineHeight.title,
    // lineHeightHeading4: typography.lineHeight.title,
    // lineHeightHeading5: typography.lineHeight.title,
  },
  inherit: true,
  components: {
    Input: {
      controlHeight: 38,
    },
    Select: {
      controlHeight: 38,
    },
    DatePicker: {
      controlHeight: 38,
    },
    InputNumber: {
      controlHeight: 38,
    },
  },
};

const Internationalization = ({ children }: InternationalizationProps) => {
  const [localeStorage] = useLocalStorage('sbt-locale', getLocale());
  const { data: locale } = useGlobalState(['sbt-locale'], localeStorage);

  React.useEffect(() => {
    dynamicActivate(locale);
  }, [locale]);

  const antdLocale = React.useMemo(() => antdLocales[locale], [locale]);

  return (
    <I18nProvider i18n={i18n} forceRenderOnLocaleChange>
      <ConfigProvider
        locale={antdLocale}
        theme={theme}
        prefixCls={ANT_PREFIX_CLASS}
      >
        {children}
      </ConfigProvider>
    </I18nProvider>
  );
};

export default React.memo(Internationalization);
