import {
  GlobalOutlined,
  LogoutOutlined,
  SettingOutlined,
} from '@ant-design/icons';
import { t } from '@lingui/macro';
import { Dropdown, Menu } from 'antd';
import { useAuth } from 'hooks/useAuth';
import { DEFAULT_LOCALE } from 'i18n';
import { useNavigate } from 'react-router-dom';
import { useGlobalState, useLocalStorage } from 'utils/hooks';

const UserHeader = () => {
  const { logOut } = useAuth();
  const navigate = useNavigate();
  const [, setLocaleStorage] = useLocalStorage('sbt-locale', DEFAULT_LOCALE);
  const { data: locale, setData: setLocale } = useGlobalState(
    ['sbt-locale'],
    DEFAULT_LOCALE,
  );

  const userMenu = (
    <Menu className='sbt-header-menu' selectedKeys={[locale]}>
      <Menu.SubMenu
        key='language'
        icon={<GlobalOutlined />}
        title={<span>{t`Language`}</span>}
      >
        <Menu.Item
          key='vi'
          onClick={() => {
            setLocaleStorage('vi');
            setLocale('vi');
          }}
        >
          {t`Vietnamese`}
        </Menu.Item>
        <Menu.Item
          key='en'
          onClick={() => {
            setLocaleStorage('en');
            setLocale('en');
          }}
        >
          {t`English`}
        </Menu.Item>
      </Menu.SubMenu>

      <Menu.Item
        key='logout'
        icon={<LogoutOutlined />}
        onClick={async () => {
          await logOut();
          navigate('/auth/login');
        }}
      >
        {t`Logout`}
      </Menu.Item>
    </Menu>
  );

  return (
    <>
      <Dropdown
        placement='bottomRight'
        overlay={userMenu}
        arrow={{ pointAtCenter: true }}
      >
        <SettingOutlined style={{ cursor: 'pointer' }} />
      </Dropdown>
    </>
  );
};

export default UserHeader;
