import { useQueryClient } from '@tanstack/react-query';
import { useLogIn, useLogout } from 'queries';
import { LogInResponse } from 'request/user';
import { useGlobalState } from 'utils/hooks';

export function useAuth() {
  const queryClient = useQueryClient();
  const { data: isAuthenticated } = useGlobalState(
    ['isAuthenticated'],
    Boolean(localStorage.getItem('sbt-token')),
  );

  const logInMutation = useLogIn({
    onSuccess(data: LogInResponse) {
      localStorage.setItem('sbt-token', JSON.stringify(data.token));
      localStorage.setItem('user-name', JSON.stringify(data.user_name));
      window.location.reload();
    },
  });
  const logOutMutation = useLogout({
    onSuccess() {
      localStorage.removeItem('sbt-token');
      queryClient.clear();
      window.location.reload();
    },
  });

  return {
    isAuthenticated,
    isAuthenticating: logInMutation.isLoading,
    logIn: logInMutation.mutateAsync,
    logOut: logOutMutation.mutateAsync,
  };
}
