import React from 'react';
import { NavigateFunction, useNavigate } from 'react-router-dom';

/*
  Q: Why do I create this hook?
  A: `navigation` function is retrieved from react-router-dom context. This function is created new everytime context changes.
  So, there will be sometimes you get stuck in old `navigate` function and feature won't work (this can happen by JS closure mechanism or react memorization).
  To make sure that you always use the latest `navigate` function, use this hook.
  Note: Only prioritize to use this hook when something does not work as expected. E.g. https://devops.sdsdev.co.kr/jira/browse/IRVAIOPER-461
*/
export function useNavigateRef() {
  const navigate = useNavigate();
  const navigateRef = React.useRef<NavigateFunction>(navigate);

  React.useEffect(() => {
    navigateRef.current = navigate;
  }, [navigate]);

  return navigateRef;
}
