import { UserOutlined } from '@ant-design/icons';
import {
  Avatar,
  Divider,
  Dropdown,
  Layout,
  MenuProps,
  Row,
  Space,
  theme,
  Typography,
} from 'antd';
import LeftMenu from 'components/left-menu';
import { GlobalSpin } from 'components/spin';
import * as React from 'react';
import { Link } from 'react-router-dom';
import { layout } from 'theme/opus';

export const MainLayout = ({ children }: { children: React.ReactNode }) => {
  const {
    token: { colorBgContainer, colorBorder },
  } = theme.useToken();

  const userName = localStorage.getItem('user-name')
    ? JSON.parse(localStorage.getItem('user-name'))
    : '';
  const items: MenuProps['items'] = [
    // {
    //   key: 'user_management',
    //   label: <>User Management</>,
    // },
    {
      key: 'logout',
      label: <>Logout</>,
      onClick: () => {
        localStorage.clear();
        window.location.href = '/auth/login';
      },
    },
  ];

  return (
    <Layout style={{ height: '100vh' }}>
      <Layout.Header
        style={{
          height: layout.area.headerHeight,
          background: colorBgContainer,
          borderBottom: `1px solid ${colorBorder}`,
          padding: '0 16px',
        }}
      >
        <Row justify='space-between' align='middle'>
          <Link
            to='/'
            style={{
              display: 'flex',
              alignItems: 'center',
              gap: '10px',
              cursor: 'pointer',
            }}
          >
            <img src={'/sbt_icon.png'} alt='icon' width={40} height={40} />
            <Typography.Title
              level={2}
              style={{
                color: '#004F8D',
                cursor: 'default',
                marginBottom: '0',
              }}
            >
              SBT
            </Typography.Title>
          </Link>

          <Space split={<Divider type='vertical' />}>
            <Dropdown
              menu={{ items }}
              placement='bottomRight'
              arrow={{ pointAtCenter: true }}
            >
              <Space align='center'>
                <Avatar icon={<UserOutlined />} />
                <Typography.Text
                  style={{
                    maxWidth: 300,
                    verticalAlign: 'middle',
                    textAlign: 'center',
                  }}
                  ellipsis={{ tooltip: userName }}
                >
                  {userName}
                </Typography.Text>
              </Space>
            </Dropdown>
          </Space>
        </Row>
      </Layout.Header>
      <Layout.Content
        style={{
          height: `calc(100vh - ${layout.area.headerHeight})`,
          overflow: 'hidden',
        }}
      >
        <Layout
          style={{
            height: '100%',
          }}
        >
          <Layout.Sider
            style={{
              background: colorBgContainer,
              padding: 8,
              borderRight: `1px solid ${colorBorder}`,
            }}
          >
            <LeftMenu />
          </Layout.Sider>
          <Layout.Content
            style={{
              height: '100%',
              overflow: 'auto',
              padding: 16,
              background: colorBgContainer,
            }}
          >
            <React.Suspense fallback={<GlobalSpin />}>
              {children}
            </React.Suspense>
          </Layout.Content>
        </Layout>
      </Layout.Content>
    </Layout>
  );
};
