import { LockOutlined, UserOutlined } from '@ant-design/icons';
import { t } from '@lingui/macro';
import { useLingui } from '@lingui/react';
import { Button, Form, FormProps, Input, notification, Typography } from 'antd';
import { AxiosError } from 'axios';
import { DIVERGE_FORM_ICON_COLOR } from 'consts';
import { useAuth } from 'hooks/useAuth';
import { LogInPayload } from 'request';
import styled from 'styled-components';
import { getErrorMessage } from 'utils/error-handler';
import loginBgUri from '../../assets/login_bg.svg';

const StyledLoginContainer = styled.div`
  width: 100%;
  height: 100%;
  background-image: url(${loginBgUri});
  background-position: center;
  background-repeat: no-repeat;
  background-size: cover;
  display: flex;
  justify-content: center;
  align-items: center;
  position: relative;
`;

const StyledFormContainer = styled(Form)`
  .login-form__company {
    color: white;
    text-align: justify;
    font-size: 52px;
    margin-bottom: 0;
  }

  .login-form__center {
    color: #84b1e3;
    text-align: right;
    font-weight: 300;
    margin-top: 0 !important;
  }

  .login-form__project-name {
    color: white;
    text-align: right;
    font-weight: 400;
    margin-top: 0.8em !important;
  }

  .ant-form-item-control-input .ant-input-affix-wrapper {
    border: none;
  }

  .login-form__helper-text {
    color: #83afdf;
  }

  .login-form__submit-button {
    display: block;
    width: 100%;
  }

  a {
    color: white;
    text-decoration: underline;
  }
`;

export default function LoginPage() {
  const { i18n } = useLingui();
  const { logIn, isAuthenticating } = useAuth();

  const handleLogin: FormProps<LogInPayload>['onFinish'] = async (payload) => {
    try {
      await logIn(payload);
      notification.success({
        message: t(i18n)`User log in successfully`,
      });
    } catch (error) {
      if (error instanceof AxiosError) {
        notification.error({
          message: getErrorMessage(error),
        });
      }
    }
  };

  return (
    <StyledLoginContainer>
      <StyledFormContainer className='login-form' onFinish={handleLogin as any}>
        <Typography.Title level={1} className='login-form__company'>
          SAMSUNG SDS
        </Typography.Title>
        <Typography.Title level={3} className='login-form__center'>
          R&D CENTER
        </Typography.Title>
        <Typography.Title level={2} className='login-form__project-name'>
          SBT
        </Typography.Title>

        <Form.Item
          name='username'
          rules={[
            {
              required: true,
              message: t(i18n)`Please specify a username`,
              whitespace: true,
            },
          ]}
        >
          <Input
            autoComplete='username'
            prefix={<UserOutlined style={{ color: DIVERGE_FORM_ICON_COLOR }} />}
            placeholder={t(i18n)`Username`}
            size='large'
          />
        </Form.Item>

        <Form.Item
          name='password'
          rules={[
            {
              required: true,
              message: t(i18n)`Please specify a password`,
              whitespace: true,
            },
          ]}
        >
          <Input.Password
            autoComplete='current-password'
            prefix={<LockOutlined style={{ color: DIVERGE_FORM_ICON_COLOR }} />}
            placeholder={t(i18n)`Password`}
            size='large'
          />
        </Form.Item>

        <Form.Item>
          <Button
            type='primary'
            htmlType='submit'
            className='login-form__submit-button'
            size='large'
            loading={isAuthenticating}
          >
            {t(i18n)`Login`}
          </Button>
        </Form.Item>
      </StyledFormContainer>
    </StyledLoginContainer>
  );
}
