import { baseURL } from 'request/api';

export const fetchAllRequests = async (
  filterDateRange,
  filterSubsidiaries,
  filterReviewState,
  filterIncludeTests,
  page = 1,
  page_size = 20,
  max_accuracy = 100
) => {
  const startDate =
    filterDateRange && filterDateRange[0] ? filterDateRange[0] : '';
  const endDate =
    filterDateRange && filterDateRange[1] ? filterDateRange[1] : '';
  let filterStr = '';
  filterStr += `page=${page}&page_size=${page_size}&`;
  if (filterSubsidiaries) {
    filterStr += `subsidiary=${filterSubsidiaries}&`;
  }
  if (filterReviewState) {
    filterStr += `is_reviewed=${filterReviewState}&`;
  }
  if (filterIncludeTests) {
    filterStr += `includes_test=${filterIncludeTests}&`;
  }
  if (startDate && endDate) {
    filterStr += `start_date=${startDate}&end_date=${endDate}&`;
  }
  filterStr += `max_accuracy=${max_accuracy}`
  const token = localStorage.getItem('sbt-token') || '';
  const data = await fetch(`${baseURL}/ctel/request_list/?${filterStr}`, {
    method: 'GET',
    headers: {
      Authorization: `${JSON.parse(token)}`,
    },
  }).then(async (res) => {
    const data = await res.json();
    return data;
  });
  return data;
};

export const updateRevisedData = async (
  requestID: any,
  newRevisedData: any,
) => {
  // const requestID = ;
  const token = localStorage.getItem('sbt-token') || '';
  const result = await fetch(`${baseURL}/ctel/request/${requestID}/`, {
    method: 'POST',
    headers: {
      Authorization: `${JSON.parse(token)}`,
      'Content-Type': 'application/json',
    },
    body: JSON.stringify({
      reviewed_result: newRevisedData,
    }),
  }).catch((error) => {
    console.log(error);
    throw error;
  });
  if (result.status != 200) {
    throw new Error('Could not update revised data');
  }
};

export const fetchRequest = async (id) => {
  const token = localStorage.getItem('sbt-token') || '';
  const response = await fetch(`${baseURL}/ctel/request/${id}/`, {
    method: 'GET',
    headers: {
      Authorization: `${JSON.parse(token)}`,
    },
  });
  return await (
    await response.json()
  ).subscription_requests[0];
};
