// open modal show a list from local storage

import { DeleteOutlined, HistoryOutlined } from '@ant-design/icons';
import { Button, Modal, Tag } from 'antd';
import dayjs from 'dayjs';
import { useState } from 'react';
import { RecentRequest } from './const';
const RecentRequesModal = () => {
  const [showModal, setShowModal] = useState(false);
  const [listData, setListData] = useState<RecentRequest[]>([]);

  const handleShowModal = () => {
    const data = localStorage.getItem('recent-request');
    if (data) {
      setListData(JSON.parse(data));
    }
    setShowModal(true);
  };

  const handleCloseModal = () => {
    setShowModal(false);
  };

  return (
    <div>
      <Button onClick={handleShowModal} icon={<HistoryOutlined />}>Recents </Button>
      <Modal
        style={{ overflow: 'auto' }}
        footer={
          <div style={{textAlign: 'left'}}>
            {listData.length > 0 ? (
              <Button
                danger
                style={{ marginTop: '16px' }}
                icon={<DeleteOutlined />}
                onClick={() => {
                  if (
                    window.confirm(
                      'Are you sure you want to remove all recent requests?',
                    )
                  ) {
                    localStorage.removeItem('recent-request');
                    setListData([]);
                  }
                }}
              >
                {' '}
                Remove all
              </Button>
            ) : (
              <p>No recent requests</p>
            )}
          </div>
        }
        width={600}
        height={600}
        open={showModal}
        title='Recent Requests'
        onCancel={handleCloseModal}
      >
        {listData.map((item, index) => (
          <div style={{ marginBottom: '4px' }}>
            <a
              href={`?request_id=${item.request_id}`}
              target='_blank'
              key={index}
            >
              <Tag color='blue'>{dayjs(item.time).format('hh:mm - DD/MM')}</Tag>
              <span>{item.request_id}</span>
            </a>
          </div>
        ))}
      </Modal>
    </div>
  );
};

export default RecentRequesModal;
