import { useMutation, useQueryClient } from '@tanstack/react-query';
import { CustomUseMutationOptions, User } from 'models';
import {
  activateUser,
  changePassword,
  changePasswordForOther,
  deactivateUser,
  logIn,
  logOut,
  registerAccount,
  updateUser,
} from 'request';

export function useLogIn(options?: CustomUseMutationOptions) {
  return useMutation({
    mutationFn: logIn,
    ...options,
  });
}

export function useLogout(
  options?: CustomUseMutationOptions<any, unknown, void>,
) {
  return useMutation({
    mutationFn: logOut,
    ...options,
  });
}

export function useRegister(options?: CustomUseMutationOptions) {
  return useMutation({
    mutationFn: registerAccount,
    ...options,
  });
}

export function useUpdateUser(options?: CustomUseMutationOptions<User>) {
  const queryClient = useQueryClient();
  return useMutation({
    mutationFn: updateUser,
    onSuccess() {
      queryClient.invalidateQueries(['users']);
      queryClient.invalidateQueries(['user-self']);
    },
    ...options,
  });
}

export function useActivateUser(options?: CustomUseMutationOptions<User>) {
  const queryClient = useQueryClient();
  return useMutation({
    mutationFn: activateUser,
    onSuccess() {
      queryClient.invalidateQueries(['users']);
    },
    ...options,
  });
}

export function useDeactivateUser(options?: CustomUseMutationOptions<User>) {
  const queryClient = useQueryClient();
  return useMutation({
    mutationFn: deactivateUser,
    onSuccess() {
      queryClient.invalidateQueries(['users']);
    },
    ...options,
  });
}

export function useChangePassword(options?: CustomUseMutationOptions) {
  const queryClient = useQueryClient();
  return useMutation({
    mutationFn: changePassword,
    onSuccess() {
      localStorage.removeItem('sbt-token');
      queryClient.setQueryData(['isAuthenticated'], false);
    },
    ...options,
  });
}

export function useChangePasswordForOther(options?: CustomUseMutationOptions) {
  return useMutation({
    mutationFn: changePasswordForOther,
    ...options,
  });
}
