import { User } from 'models';
import { API, PublicAPI } from './api';

export async function getSelf() {
  const response = await API.get<User>('/users/self');
  return response.data;
}

export async function getSession() {
  const response = await API.get<{ success: boolean; valid: boolean }>(
    '/check_session',
  );
  return response.data;
}

export interface LogInPayload {
  username: string;
  password: string;
}

export interface LogInResponse {
  user_id: number;
  user_name: string;
  token: string;
}

export async function logIn(payload: LogInPayload) {
  const response = await PublicAPI.post<LogInResponse>('/ctel/login/', payload);
  return response.data;
}

export async function logOut() {
  const response = await API.post('/auth/logout');
  return response.data;
}

export interface RegisterPayload {
  email: string;
  username: string;
  fist_name: string;
  last_name: string;
  password1: string;
  password2: string;
}

export async function registerAccount(payload: RegisterPayload) {
  const response = await PublicAPI.post('/auth/register', payload);
  return response.data;
}

export async function updateUser(
  partialUser: Omit<Partial<User>, 'id'> & { id: User['id'] },
) {
  const response = await API.patch<User>(
    `/users/${partialUser.id}`,
    partialUser,
  );
  return response.data;
}

export function activateUser(id: User['id']) {
  return updateUser({
    id,
    is_active: true,
  });
}

export function deactivateUser(id: User['id']) {
  return updateUser({
    id,
    is_active: false,
  });
}

export interface ChangePassOtherPayload {
  username: User['username'];
  password: string;
}
export async function changePasswordForOther(payload: ChangePassOtherPayload) {
  const response = await API.post('/auth/password/change_for_user', {
    username: payload.username,
    new_password: payload.password,
  });
  return response.data;
}

export interface ChangePassPayload {
  old_password: string;
  new_password1: string;
  new_password2: string;
}
export async function changePassword(payload: ChangePassPayload) {
  const response = await API.post('/auth/password/change', payload);
  return response.data;
}
