import { ForbiddenResult } from 'components/results';
import { useAuth } from 'hooks/useAuth';
import { Navigate } from 'react-router-dom';

export function PublicRoute({ element }: { element: React.ReactNode }) {
  const { isAuthenticated } = useAuth();

  if (isAuthenticated) {
    return <Navigate to='/dashboard' />;
  }

  return <>{element}</>;
}

export function PrivateRoute({
  element,
  additionalGuards = [],
}: {
  element: React.ReactNode;
  additionalGuards?: Array<boolean | (() => boolean)>;
}) {
  const { isAuthenticated } = useAuth();
  const isGuardsPassed = additionalGuards.every((item) =>
    typeof item === 'boolean' ? item : item(),
  );

  if (!isAuthenticated) {
    return <Navigate to='/auth/login' />;
  }

  // if (isFetchingUser) {
  //   return <GlobalSpin />;
  // }

  if (!isGuardsPassed) {
    return <ForbiddenResult />;
  }

  return <>{element}</>;
}
