export const IMAGE_MIME_TYPES = [
  'image/bmp',
  'image/png',
  'image/jpeg',
  'image/gif',
];
export const IMAGE_EXTENSIONS = ['.bmp', '.jpeg', '.jpg', '.png', '.gif'];

export const MODEL_FILE_EXTENSIONS = ['.onnx', '.pth', '.zip', '.h5'];
export const PROGRAM_FILE_EXTENSIONS = ['.exe', '.zip'];
export const FILE_SIZE_LIMIT = 4096 * 1024 * 1024;
export const IMAGE_SIZE_LIMIT = 200 * 1024 * 1024;
export const MODEL_FILE_SIZE_LIMIT = 4096 * 1024 * 1024;
export const PROGRAM_FILE_SIZE_LIMIT = 4096 * 1024 * 1024;

export function isImageFile(file: File) {
  return IMAGE_MIME_TYPES.includes(file.type);
}

export function isProgramFile(file: File) {
  return PROGRAM_FILE_EXTENSIONS.some((ext) => {
    return file.name.toLowerCase().endsWith(ext);
  });
}

export function isModelFile(file: File) {
  return MODEL_FILE_EXTENSIONS.some((ext) => {
    return file.name.toLowerCase().endsWith(ext);
  });
}

export function isZipFile(file: File) {
  return file.name.toLowerCase().endsWith('.zip');
}

export function isFileUnderLimit(file: File, limit: number = FILE_SIZE_LIMIT) {
  return file.size <= limit;
}
