import { t } from '@lingui/macro';

export const EMAIL_PATTERN =
  /^[a-zA-Z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/g;

export const DOMAIN_PATTERN =
  /^(?:[a-zA-Z0-9](?:[a-zA-Z0-9-_]{0,61}[A-Za-z0-9])?\.)+[A-Za-z0-9][A-Za-z0-9-_]{0,61}[A-Za-z]$/i;

export const MIN_PASSWORD_LENGTH = 10;
export const MAX_PASSWORD_LENGTH = 128;
export const MAX_USERNAME_LENGTH = 150;
export const MAX_STRING_LENGTH = 255;
export const MAX_EMAIL_LENGTH = 254;

const validationPatterns = {
  validatePasswordMinLength: {
    pattern: new RegExp(`(?=.{${MIN_PASSWORD_LENGTH},})`),
    get message() {
      return t`Password must have at least ${MIN_PASSWORD_LENGTH} characters`;
    },
  },

  validatePasswordMaxLength: {
    pattern: new RegExp(`^[\\s\\S]{0,${MAX_PASSWORD_LENGTH}}$`),
    get message() {
      return t`Password must not have more than ${MAX_PASSWORD_LENGTH} characters`;
    },
  },

  passwordContainsNumericCharacters: {
    pattern: /(?=.*[0-9])/,
    get message() {
      return t`Password must have at least 1 numeric characters`;
    },
  },

  passwordContainsUpperCaseCharacter: {
    pattern: /(?=.*[A-Z])/,
    get message() {
      return t`Password must have at least 1 uppercase alphabetical character`;
    },
  },

  passwordContainsLowerCaseCharacter: {
    pattern: /(?=.*[a-z])/,
    get message() {
      return t`Password must have at least 1 lowercase alphabetical character`;
    },
  },

  validateUsernameMaxLength: {
    pattern: new RegExp(`^[\\s\\S]{0,${MAX_USERNAME_LENGTH}}$`),
    get message() {
      return t`Username must not have more than ${MAX_USERNAME_LENGTH} characters`;
    },
  },

  validateUsernameCharacters: {
    pattern: /^[a-zA-Z0-9@.+\-_]*$/,
    get message() {
      return t`Only characters (a-z), (A-Z), (0-9), @, ., +, -, _ are available`;
    },
  },

  validateEmail: {
    pattern: EMAIL_PATTERN,
    get message() {
      return t`Email format is not correct`;
    },
  },

  validateStringMaxLength: {
    pattern: new RegExp(`^[\\s\\S]{0,${MAX_STRING_LENGTH}}$`),
    get message() {
      return t`This field must not have more than ${MAX_STRING_LENGTH} characters`;
    },
  },

  validateFirstLastNameMaxLength: {
    pattern: new RegExp(`^[\\s\\S]{0,${MAX_USERNAME_LENGTH}}$`),
    get message() {
      return t`This field must not have more than ${MAX_USERNAME_LENGTH} characters`;
    },
  },

  validateEmailMaxLength: {
    pattern: new RegExp(`^[\\s\\S]{0,${MAX_EMAIL_LENGTH}}$`),
    get message() {
      return t`This field must not have more than ${MAX_EMAIL_LENGTH} characters`;
    },
  },
  validateDomain: {
    pattern: DOMAIN_PATTERN,
    get message() {
      return t`Please enter a valid domain`;
    },
  },
};

export default validationPatterns;
